/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * BGNEstimator.cpp
 *
 *  Created on: 2010/12/09
 *      Author: HARK Development Team
 */

#include <cmath>
#include <boost/shared_ptr.hpp>
#include <BGNEstimator.hpp>

using namespace std;
using namespace boost;

namespace NSHarkBinaural
{


BGNEstimator::BGNEstimator(unsigned int estimateCount)
    : estimateCounter(0), estimateCount(estimateCount), estimated(false)
{ }

void BGNEstimator::Estimate(LRSpectra::ptr_t lrspectra)
{
    static Logger& logger = Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME);

    if (!IsEqualSignalSize(lrspectra))
    {
        // 要素がnullか、左右の信号の要素数が異なる場合は無視
        return;
    }
    if (this->estimateCounter == 0)
    {
        // 初回のみバックグラウンドノイズのための領域を生成
        // Spectrumのコンストラクタの仕様により、各要素は0で初期化される
        logger.Fine("estimateCounter == 0 , creating bgnoiseSpectra");
        size_t len = lrspectra->Left()->Size();
        Unspectrum::ptr_t unspecl(new Unspectrum(len));
        Unspectrum::ptr_t unspecr(new Unspectrum(len));
        Spectrum::ptr_t specl(new Spectrum(len));
        Spectrum::ptr_t specr(new Spectrum(len));
        this->bgnoiseSpectra = LRSpectra::ptr_t(new LRSpectra(specl, specr));
        this->bgnoisePowerSpectra = LRUnspectra::ptr_t(new LRUnspectra(unspecl, unspecr));
    }

    Spectrum &spl = *lrspectra->Left(), &spr = *lrspectra->Right();
    Spectrum &bgspl = *this->bgnoiseSpectra->Left(), &bgspr = *this->bgnoiseSpectra->Right();
    Unspectrum &bgl = *this->bgnoisePowerSpectra->Left(), &bgr = *this->bgnoisePowerSpectra->Right();

    if (this->estimateCounter < this->estimateCount)
    {
        LogFine(logger, boost::format("estimating noise: estimateCounter == %d") % this->estimateCounter);
        this->estimateCounter++;
        for (size_t i = 0; i < spl.Size(); ++i)
        {
            bgl.At(i) += abs(spl.At(i));
            bgspl.At(i) += spl.At(i);
            bgr.At(i) += abs(spr.At(i));
            bgspr.At(i) += spr.At(i);
        }
    }
    else if (this->estimateCounter == this->estimateCount)
    {
        this->estimateCounter++;
        for (size_t i = 0; i < spl.Size(); ++i)
        {
            bgl.At(i) = 20 * log10(bgl.At(i) / this->estimateCount + 1e-100);
            bgspl.At(i) /= this->estimateCount;
            bgr.At(i) = 20 * log10(bgr.At(i) / this->estimateCount + 1e-100);
            bgspr.At(i) /= this->estimateCount;
        }
        LogFine(logger, "got background noise");
        this->estimated = true;
    }
}

LRSpectra::ptr_t BGNEstimator::BGNoiseSpectra()
{
    if (this->Estimated())
    {
        return this->bgnoiseSpectra;
    }
    return LRSpectra::ptr_t(); // return null pointer
}

LRUnspectra::ptr_t BGNEstimator::BGNoise()
{
    if (this->Estimated())
    {
        return this->bgnoisePowerSpectra;
    }
    return LRUnspectra::ptr_t(); // return null pointer
}

}
